BEGIN TRANSACTION;
CREATE TABLE "person" (
	"id"	INTEGER NOT NULL,
	"name"	NVARCHAR(80) NOT NULL,
	"age"	INTEGER,
	PRIMARY KEY("id" AUTOINCREMENT)
);
CREATE TABLE "cat" (
	"id"	INTEGER NOT NULL,
	"name"	NVCHAR(80) NOT NULL,
	"owner_id"	INTEGER,
	PRIMARY KEY("id" AUTOINCREMENT),
	FOREIGN KEY("owner_id") REFERENCES "person"("id") ON DELETE CASCADE
);
CREATE INDEX index_person
ON person ( id COLLATE NOCASE );
CREATE INDEX index_cat
ON cat ( id COLLATE NOCASE );
COMMIT;
